#!/usr/bin/env python

# Script to automatically close bug reports reported on
#   lp:~openerp/openobject-client-web
# because development effort moved to the new openerp-web project.

import launchpadlib
from launchpadlib.launchpad import Launchpad
from launchpadlib.credentials import Credentials
import os

# List of bug status:
#   [ 'New', 'Incomplete', 'Invalid', 'Opinion',
#     'Confirmed', 'Won't Fix', 'Triaged',
#     'In Progress', 'Fix Committed', 'Fix Released']
OPEN_BUGS_STATUS = ['Confirmed', 'Triaged', 'In Progress']
NEW_BUGS_STATUS = OPEN_BUGS_STATUS + ['New', 'Fix Committed']

# List of bug importances:
#  [ 'Critical', 'High', 'Medium', 'Low', 'Undecided',
#    'Wishlist']

# List of merge status:
#   [ 'Work in progress', 'Needs review', 'Code failed to merge', 'Queued',
#     'Superseded', 'Merged', 'Approved', 'Rejected' ]
NEW_MERGE_STATUS = ['Needs review','Code failed to merge', 'Approved']

cachedir = os.path.expanduser("~/.launchpadlib/cache")

APP_ID = 'openerp-manual-bug-process'
LP_PROD = launchpadlib.uris.lookup_service_root('production')
LP_TEST = launchpadlib.uris.lookup_service_root('staging')


def close_bug(bug_task):
    msg = bug_task.bug.newMessage(subject="OpenERP Majordomo Message",
                                  content="""
**automated message**
Thanks a lot for reporting this bug and contributing to OpenERP. Unfortunately the OpenERP Web Client project is not going to be developed further after the 6.0 series. As of 6.1 a newer and better web frontend known as the OpenERP Web project[1] will be available.
Our R&D developers normally work in Launchpad to fix bugs for the next release, but there will be no next release for the current OpenERP Web Client. As our resources are limited, all further R&D efforts from the Web team are dedicated to finish the new 6.1 OpenERP Web project[1].
This bug will be closed as "Won't Fix" to show that R&D won't be able to work on it.

Bugs affecting customers in production of course continue to be handled via the OpenERP Enterprise (OPW) maintenance service, and this is the recommended way to have them corrected directly in the 6.0 LTS stable branch.
You can find more details about all of this in the FAQ of our Bug Management Policy documentation[2].

If you are interested, you should soon be able to beta-test the new OpenERP Web 6.1 (this will be announced on OpenERP Community channels).

Thank you for your contributions and for your understanding!

[1] https://launchpad.net/openerp-web
[2] http://bit.ly/openerp-bugs-faq (FAQ #4)""")
    bug_task.status = "Won't Fix"
    bug_task.lp_save()


if __name__ == '__main__':

    lp = Launchpad.login_with(APP_ID, LP_PROD, cachedir)

    lp_source = lp.people['openerp']
    lp_source = lp.projects['openobject-client-web']

    matching_bugtasks = lp_source.searchTasks(
                            status=['Confirmed','Triaged'],
                            importance=['Medium', 'Low', 'Wishlist'],
                    )

    # Warning: Launchpad's collections do not support len(), so unfortunately
    #          we need to walk the results to count them!
    matching_bugtasks = [bug_task for bug_task in matching_bugtasks]
    for bug_task in matching_bugtasks:
        print bug_task, bug_task.importance, bug_task.status, bug_task.bug
        close_bug(bug_task)

    print "Found %s matching bugs" % len(matching_bugtasks)
