from launchpadlib.launchpad import Launchpad, EDGE_SERVICE_ROOT
from launchpadlib.credentials import Credentials
import threading
from datetime import datetime
import os
from optparse import OptionParser

def export_csv(fname, fields, result, write_title=False):
    import csv
    import types
    fp = file(fname, 'wb+')
    writer = csv.writer(fp, quoting=csv.QUOTE_ALL)
    fields.sort()
    if write_title:
        writer.writerow(fields)

    for key, values in result.items():
        row = [key]
        writer.writerow(row)
        for key2, values2 in values.items():
            row = [key2]
            for field in fields:
                if not field:
                    continue
                if field not in values2:
                    d = 0
                else:
                    d = values2[field]
                if type(d)==types.StringType:
                    row.append(d.replace('\n','0').replace('\t','0'))
                else:
                    row.append(d or 0)
            writer.writerow(row)
        
    fp.close()
    return True
    

ALL_BUGS_STATUS =  ["New", "Incomplete", "Opinion", "Invalid", "Won't Fix", "Expired", "Confirmed", "Triaged", "In Progress", "Fix Committed", "Fix Released", "Incomplete (with response)", "Incomplete (without response)"]
ALL_MERGE_STATUS = ['Needs review','Code failed to merge', 'Work in progress', 'Approved', 'Queued', 'Superseded', 'Merged', 'Rejected' ]

BUGS_QUALITY_STATUS = ['New', 'Confirmed', 'Triaged', 'Closed', 'Incomplete', 'Open', 'Invalid']
PROJECTS = ['openobject-addons', 'openobject-server', 'openobject-client', 'openobject-client-web', 'openobject-doc']

BUGS_STATUS = ['Open', 'Assigned', 'Closed', 'Re-Open']
DEV_STATUS = ['created', 'submited', 'merged', 'resubmit'] #'diff-new-lines', 'diff-remove-lines', 'diff-lines', 'diff-modified-files','unknown'
DEV_QUALITY_STATUS = ['open', 'approved', 'be-merged', 'need-fixing', 'need-information', 'disapproved']
IMPORTANCE = ['Critical', 'High', 'Medium', 'Low', 'Undecided']
ASSIGNEES = ['openerp-dev-framework', 'openerp-dev-gtk', 'openerp-dev-web', 'openerp-dev-addons1', 'openerp-dev-addons2', 'openerp-dev-addons3', 'openerp-opw']

TEAMS = ['openerp','openerp-dev','openobject-training', 'openerp-opw']

class lpServer(threading.Thread):
    host = 'localhost'
    port = 8200
    cachedir = os.path.expanduser("~/.launchpadlib/cache")
    launchpad = False
    login = 'hmo-tinyerp'
    file_name = False
    from_date = False
    fields = ['']

    def __init__(self, login, month, file_name):
        super(lpServer, self).__init__()
        self.launchpad = self.get_lp()
        self.file_name = file_name
        self.login = login
        ct = datetime.now()
        year = ct.year
        month = ct.month - month
        if month < 0:
            year = ct.year-1
            month = ct.month + 12 - month
        day = 1
        self.from_date = datetime(year, month, day)
        
    
    def get_lp(self):
        return Launchpad.login_with(self.login, EDGE_SERVICE_ROOT, self.cachedir)

    def get_lp_bug_stat(self):
        bugs_dict = {}

        for assignee in ASSIGNEES:
            bugs_dict[assignee] = self.get_lp_bugs_assignee(assignee)

        total_dict = {}
        for assignee in bugs_dict:
            for status in bugs_dict[assignee]:        
                if status not in total_dict:
                    total_dict[status] = {}
                total_status_dict = total_dict[status]
                for week in bugs_dict[assignee][status]:
                    if week not in total_status_dict:
                        total_status_dict[week] = 0
                    
                    total_status_dict[week]+=bugs_dict[assignee][status][week]
        bugs_dict['total'] = total_dict
        return bugs_dict

    def get_lp_bugs_assignee(self, assignee):
        lp_source = self.launchpad.people[assignee]
        assignee_bugs = lp_source.searchTasks(importance=IMPORTANCE, status=ALL_BUGS_STATUS)
        assignee_dict = {}
        weeks = self.fields
        for status in BUGS_STATUS:
            lstatus = status.lower()
            if lstatus not in assignee_dict:
                assignee_dict[lstatus] = {}
            
        for assignee_bug in assignee_bugs:
            td = assignee_bug.date_created
            fd = self.from_date
            if datetime(td.year, td.month, td.day) <= datetime(fd.year, fd.month, fd.day):
                continue
                        
            assigned_dict = assignee_dict['assigned']
            reopen_dict = assignee_dict['re-open']
            closed_dict = assignee_dict['closed']
            open_dict = assignee_dict['open']


            if assignee_bug.date_assigned:
                week = 'week-%s' % (assignee_bug.date_assigned.strftime('%U'))
                if week not in weeks: weeks.append(week)
                if week not in assigned_dict:
                    assigned_dict[week] = 0
                assigned_dict[week] += 1
            if assignee_bug.date_in_progress:
                week = 'week-%s' % (assignee_bug.date_in_progress.strftime('%U'))
                if week not in weeks: weeks.append(week)
                if week not in open_dict:
                    open_dict[week] = 0
                open_dict[week] += 1
            if assignee_bug.date_fix_released:
                week = 'week-%s' % (assignee_bug.date_fix_released.strftime('%U'))
                if week not in weeks: weeks.append(week)
                if week not in closed_dict:
                    closed_dict[week] = 0
                closed_dict[week] += 1

            

            if assignee_bug.date_left_closed:
                week = 'week-%s' % (assignee_bug.date_left_closed.strftime('%U'))
                if week not in weeks: weeks.append(week)
                if week not in reopen_dict:
                    reopen_dict[week] = 0
                reopen_dict[week] += 1

           
         
        return assignee_dict

    def get_lp_bug_quality_stat(self):
        bugs_quality_control_dict = {}
        for project in PROJECTS:
            bugs_quality_control_dict[project] = self.get_lp_bugs_project(project)

        total_dict = {}
        for project in bugs_quality_control_dict:
            for status in bugs_quality_control_dict[project]:        
                if status not in total_dict:
                    total_dict[status] = {}
                total_status_dict = total_dict[status]
                for week in bugs_quality_control_dict[project][status]:
                    if week not in total_status_dict:
                        total_status_dict[week] = 0
                    
                    total_status_dict[week]+=bugs_quality_control_dict[project][status][week]
        bugs_quality_control_dict['total'] = total_dict
        return bugs_quality_control_dict

    def get_lp_bugs_project(self, project):
        lp_source = self.launchpad.projects[project]
        project_bugs = lp_source.searchTasks(status=ALL_BUGS_STATUS)
        project_dict = {}
        weeks = self.fields
        for status in BUGS_QUALITY_STATUS:
            lstatus = status.lower()
            if lstatus not in project_dict:
                project_dict[lstatus] = {}
        
        for project_bug in project_bugs:
            td = project_bug.date_created
            fd = self.from_date
            if datetime(td.year, td.month, td.day) <= datetime(fd.year, fd.month, fd.day):
                continue
            
            new_dict = project_dict['new']
            confirm_dict = project_dict['confirmed']
            triaged_dict = project_dict['triaged']
            closed_dict = project_dict['closed']
            incomplete_dict = project_dict['incomplete']
            invalid_dict = project_dict['invalid']
            open_dict = project_dict['open']
            if project_bug.date_created:
                week = 'week-%s' % (project_bug.date_created.strftime('%U'))
                if week not in weeks: weeks.append(week)
                if week not in new_dict:
                    new_dict[week] = 0
                new_dict[week] += 1
            if project_bug.date_confirmed:
                week = 'week-%s' % (project_bug.date_confirmed.strftime('%U'))
                if week not in weeks: weeks.append(week)
                if week not in confirm_dict:
                    confirm_dict[week] = 0
                confirm_dict[week] += 1
            if project_bug.date_fix_released:
                week = 'week-%s' % (project_bug.date_fix_released.strftime('%U'))
                if week not in weeks: weeks.append(week)
                if week not in closed_dict:
                    closed_dict[week] = 0
                closed_dict[week] += 1

            if not project_bug.date_fix_released and project_bug.date_closed:
                week = 'week-%s' % (project_bug.date_closed.strftime('%U'))
                if week not in weeks: weeks.append(week)
                if week not in invalid_dict:
                    invalid_dict[week] = 0
                invalid_dict[week] += 1

            if project_bug.date_in_progress:
                week = 'week-%s' % (project_bug.date_in_progress.strftime('%U'))
                if week not in weeks: weeks.append(week)
                if week not in open_dict:
                    open_dict[week] = 0
                open_dict[week] += 1

            if project_bug.date_incomplete:
                week = 'week-%s' % (project_bug.date_incomplete.strftime('%U'))
                if week not in weeks: weeks.append(week)
                if week not in incomplete_dict:
                    incomplete_dict[week] = 0
                incomplete_dict[week] += 1

            if project_bug.date_triaged:
                week = 'week-%s' % (project_bug.date_triaged.strftime('%U'))
                if week not in weeks: weeks.append(week)
                if week not in triaged_dict:
                    triaged_dict[week] = 0
                triaged_dict[week] += 1
     
        return project_dict  

    def get_lp_merge_stat(self):
        development_dict = {}
        MEMBERS = []
        #for team in TEAMS:
        #    lp_team = self.launchpad.people[team]
        #    for member in lp_team.members_details:
        #        if member.member.name not in MEMBERS:
        #            MEMBERS.append(member.member.name)

        
        merge_dict = self.get_lp_merge_proposal()
        development_dict = {}
        development_quality_dict = {}
        for member in merge_dict:
            if member not in development_dict:
                development_dict[member] = {}
            if member not in development_quality_dict:
                development_quality_dict[member] = {}
            for status in merge_dict[member]:
                if status in DEV_STATUS:
                    development_dict[member][status] = merge_dict[member][status]
                if status in DEV_QUALITY_STATUS:
                    development_quality_dict[member][status] = merge_dict[member][status]

        total_dict = {}
        for member in development_dict:
            for status in development_dict[member]:        
                if status not in total_dict:
                    total_dict[status] = {}
                total_status_dict = total_dict[status]
                for week in development_dict[member][status]:
                    if week not in total_status_dict:
                        total_status_dict[week] = 0
                    total_status_dict[week]+=development_dict[member][status][week]

        development_dict['total'] = total_dict

        total_quality_dict = {}
        for member in development_quality_dict:
            for status in development_quality_dict[member]:        
                if status not in total_quality_dict:
                    total_quality_dict[status] = {}
                total_status_dict = total_quality_dict[status]
                for week in development_quality_dict[member][status]:
                    if week not in total_status_dict:
                        total_status_dict[week] = 0
                    total_status_dict[week]+=development_quality_dict[member][status][week]

        development_quality_dict['total'] = total_quality_dict
        return development_dict, development_quality_dict

    def get_lp_merge_proposal(self):

        def get_temp_member_dict():
            temp_member_dict = {}
            for status in DEV_STATUS+DEV_QUALITY_STATUS:
                if status not in temp_member_dict:
                    temp_member_dict[status] = {}
            return temp_member_dict
        
        weeks = self.fields
        res_member_dict = {}
        for team in TEAMS:
            lp_team = self.launchpad.people[team]
            for branch in lp_team.getBranches(status=['Experimental', 'Development', 'Mature', 'Merged', 'Abandoned']):
                
                #if branch.owner.name != 'openerp-dev':
                #    continue
                if branch.registrant.name not in res_member_dict:
                    res_member_dict[branch.registrant.name] = get_temp_member_dict()

                member_dict = res_member_dict[branch.registrant.name]
                td = branch.date_created
                fd = self.from_date
                if datetime(td.year, td.month, td.day) <= datetime(fd.year, fd.month, fd.day):
                    continue
                created_dict = member_dict['created']
                week = 'week-%s' % (branch.date_created.strftime('%U'))
                if week not in weeks: weeks.append(week)
                if week not in created_dict:
                    created_dict[week] = 0
                created_dict[week] += 1
                res_member_dict[branch.registrant.name] = member_dict
                print branch.registrant.name, res_member_dict[branch.registrant.name]
                for proposal in branch.getMergeProposals(status=ALL_MERGE_STATUS):
                    if proposal.registrant.name not in res_member_dict:
                        res_member_dict[proposal.registrant.name] = get_temp_member_dict()

                    member_dict = res_member_dict[proposal.registrant.name]
                    td = proposal.date_created
                    fd = self.from_date
                    if datetime(td.year, td.month, td.day) <= datetime(fd.year, fd.month, fd.day):
                        continue

                    submited_dict = member_dict['submited']
                    #diff_new_lines_dict = member_dict['diff-new-lines']
                    #diff_remove_lines_dict = member_dict['diff-remove-lines']
                    #diff_lines_dict = member_dict['diff-lines']
                    #diff_modified_files_dict = member_dict['diff-modified-files']
                    merge_dict = member_dict['merged']
                    
                    preview_diff = proposal.preview_diff_link and self.launchpad.load(proposal.preview_diff_link) or None
                    review_links = self.launchpad.load(proposal.all_comments_collection_link)

                    week = 'week-%s' % (proposal.date_created.strftime('%U'))
                    if week not in weeks: weeks.append(week)
                    if week not in submited_dict:
                        submited_dict[week] = 0
                        #diff_new_lines_dict[week] = 0
                        #diff_remove_lines_dict[week] = 0
                        #diff_lines_dict[week] = 0
                        #diff_modified_files_dict[week] = 0

                    submited_dict[week] += 1
                    #diff_new_lines_dict[week] += preview_diff and preview_diff.added_lines_count or 0
                    #diff_remove_lines_dict[week] += preview_diff and preview_diff.removed_lines_count or 0
                    #diff_lines_dict[week] += preview_diff and preview_diff.diff_lines_count or 0
                    #diff_modified_files_dict[week] += preview_diff and (preview_diff.diffstat and len(preview_diff.diffstat.keys()) or 0) or 0

                    res_member_dict[proposal.registrant.name] = member_dict
                    if proposal.date_merged:
                        week = 'week-%s' % (proposal.date_merged.strftime('%U'))
                        if week not in weeks: weeks.append(week)
                        if week not in merge_dict:
                            merge_dict[week] = 0
                        merge_dict[week] += 1
                        res_member_dict[proposal.registrant.name] = member_dict
                        if proposal.merge_reporter:
                            if proposal.merge_reporter.name not in res_member_dict:
                                res_member_dict[proposal.merge_reporter.name] = get_temp_member_dict()
                            member_dict = res_member_dict[proposal.merge_reporter.name]
                            be_merge_dict = member_dict['be-merged']
                            if week not in be_merge_dict:
                                be_merge_dict[week] = 0
                            be_merge_dict[week] += 1
                            res_member_dict[proposal.merge_reporter.name] = member_dict
                        

                    if proposal.date_review_requested and proposal.reviewer:
                        if proposal.reviewer.name not in res_member_dict:
                            res_member_dict[proposal.reviewer.name] = get_temp_member_dict()
                        member_dict = res_member_dict[proposal.reviewer.name]
                        open_dict = member_dict['open']
                        week = 'week-%s' % (proposal.date_review_requested.strftime('%U'))
                        if week not in weeks: weeks.append(week)
                        if week not in open_dict:
                            open_dict[week] = 0
                        open_dict[week] += 1
                        res_member_dict[proposal.reviewer.name] = member_dict

                    for review in proposal.all_comments_collection:
                        review_date = review.date_created
                        if review.author.name not in res_member_dict:
                            res_member_dict[review.author.name] = get_temp_member_dict()
                        member_dict = res_member_dict[review.author.name]
                        review_dict = False
                        week = 'week-%s' % (review_date.strftime('%U'))
                        if week not in weeks: weeks.append(week)
                        if review.vote == 'Resubmit':
                            review_dict = member_dict['resubmit']
                        if review.vote == 'Approve':
                            review_dict = member_dict['approved']
                        if review.vote == 'Needs Fixing':
                            review_dict = member_dict['need-fixing']
                        if review.vote=='Needs Information':
                            review_dict = member_dict['need-information']
                        if review.vote=='Disapprove':
                            review_dict = member_dict['disapproved']

                        if review_dict:
                            if week not in review_dict:
                                review_dict[week] = 0
                            review_dict[week] += 1
                        res_member_dict[review.author.name] = member_dict
                    print 'proposal link:', proposal.registrant.name, res_member_dict[proposal.registrant.name]
        return res_member_dict

    def get_lp_merge_quality_stat(self):
        MEMBERS = []
        for team in TEAMS:
            lp_team = self.launchpad.people[team]
            for member in lp_team.members_details:
                if member.member.name not in MEMBERS:
                    MEMBERS.append(member.member.name)

        
        development_dict = {}
        for member in MEMBERS:
            development_dict[member] = self.get_lp_merge_proposal_project(project, member)

        total_dict = {}
        for member in development_dict:
            for status in development_dict[member]:        
                if status not in total_dict:
                    total_dict[status] = {}
                total_status_dict = total_dict[status]
                for week in development_dict[member][status]:
                    if week not in total_status_dict:
                        total_status_dict[week] = 0
                    total_status_dict[week]+=development_dict[member][status][week]

        development_dict['total'] = total_dict
        return development_dict

    def get_lp_merge_proposal_project(self, member):
        
        member_dict = {'open':{}, 'approved':{}, 'merged':{}, 'need-fixing':{}, 'need-information':{}, 'disapproved':{}}
        weeks = self.fields
        for project in PROJECTS:
            lp_project = self.launchpad.projects[project]
            for proposal in lp_project.getMergeProposals(status=ALL_MERGE_STATUS):
                td = proposal.date_created
                fd = self.from_date
                if datetime(td.year, td.month, td.day) <= datetime(fd.year, fd.month, fd.day):
                    continue

                open_dict = member_dict['open']
                approved_dict = member_dict['approved']
                need_fixing_dict = member_dict['need-fixing']
                need_information_dict = member_dict['need-information']
                disapproved_dict = member_dict['disapproved']
                merge_dict = member_dict['merged']

                review_links = self.launchpad.load(proposal.all_comments_collection_link)

                reviewer = proposal.reviewer_link and self.launchpad.load(proposal.reviewer_link)  or False
                if proposal.date_review_requested and reviewer and reviewer.name == member:
                    week = 'week-%s' % (proposal.date_review_requested.strftime('%U'))
                    if week not in weeks: weeks.append(week)
                    if week not in open_dict:
                        open_dict[week] = 0
                    open_dict[week] += 1

                merge_reporter = proposal.merge_reporter_link and self.launchpad.load(proposal.merge_reporter_link)  or False
                
           
                if proposal.date_merged and merge_reporter and merge_reporter.name==member:
                    week = 'week-%s' % (proposal.date_merged.strftime('%U'))
                    if week not in weeks: weeks.append(week)
                    if week not in merge_dict:
                        merge_dict[week] = 0
                    merge_dict[week] += 1

                for review in proposal.all_comments_collection:
                    if review.author.name != member:
                        continue
                    review_date = review.date_created
                    week = 'week-%s' % (review_date.strftime('%U'))
                    if week not in weeks: weeks.append(week)
                    
                    if review.vote=='Approve':
                        if week not in approved_dict:
                            approved_dict[week] = 0
                        approved_dict[week] += 1

                    if review.vote=='Needs Fixing':
                        if week not in need-fixing_dict:
                            need_fixing_dict[week] = 0
                        need_fixing_dict[week] += 1

                    if review.vote=='Needs Information':
                        if week not in need-information_dict:
                            need_information_dict[week] = 0
                        need_information_dict[week] += 1

                    if review.vote=='Disapprove':
                        if week not in disapproved_dict:
                            disapproved_dict[week] = 0
                        disapproved_dict[week] += 1
        return member_dict

    def run(self):
        fields = ['', 'week-1', 'week-2']
        res = {'development':
                    {
                        'rpa-openerp':{
                            'created':{'week-1':00, 'week-2':00},
                            'submited':{'week-1':00, 'week-2':00},
                            'diff-new-lines':{'week-1':00, 'week-2':00},
                            'diff-remove-lines':{'week-1':00, 'week-2':00},
                            'diff-lines':{'week-1':00, 'week-2':00},
                            'diff-modified-files':{'week-1':00, 'week-2':00},
                            'merged':{'week-1':00, 'week-2':00},
                            'resubmit':{'week-1':00, 'week-2':00},
                            
                        },
                        'rha-openerp':{
                            'created':{'week-1':00, 'week-2':00},
                            'submited':{'week-1':00, 'week-2':00},
                            'diff-new-lines':{'week-1':00, 'week-2':00},
                            'diff-remove-lines':{'week-1':00, 'week-2':00},
                            'diff-lines':{'week-1':00, 'week-2':00},
                            'diff-modified-files':{'week-1':00, 'week-2':00},
                            'merged':{'week-1':00, 'week-2':00},
                            'resubmit':{'week-1':00, 'week-2':00},
                        },
                        'total':{
                            'created':{'week-1':00, 'week-2':00},
                            'submited':{'week-1':00, 'week-2':00},
                            'diff-new-lines':{'week-1':00, 'week-2':00},
                            'diff-remove-lines':{'week-1':00, 'week-2':00},
                            'diff-lines':{'week-1':00, 'week-2':00},
                            'diff-modified-files':{'week-1':00, 'week-2':00},
                            'merged':{'week-1':00, 'week-2':00},
                            'resubmit':{'week-1':00, 'week-2':00},
                        },
                    }
                }
        bugs_dict = self.get_lp_bug_stat()
        export_csv(self.file_name+'bugs.csv', self.fields, bugs_dict, True)

        bugs_quality_dict = self.get_lp_bug_quality_stat()
        export_csv(self.file_name+'bugs_quality.csv', self.fields, bugs_quality_dict, True)

        dev_dict, dev_quality_dict = self.get_lp_merge_stat()
        export_csv(self.file_name+'merge.csv', self.fields, dev_dict, True)
        export_csv(self.file_name+'merge_quality.csv', self.fields, dev_quality_dict, True)
        
        #export_csv(self.file_name, fields, res, True)
        return True


if __name__ == '__main__':
    login = ''
    month = ''
    file_name = ''

    parser = OptionParser()
    parser.add_option("--login", action="store", dest="login")
    parser.add_option("--month", action="store", dest="month")
    parser.add_option("--file_name", action="store", dest="file_name")

    (opts, args) = parser.parse_args()

    login = opts.login
    month = opts.month
    file_name = opts.file_name

    if not login:
        login = raw_input('Launchpad login:')
    if not month:
        month = raw_input('Number of Last Month (Report Since):')
    if not file_name:
        file_name = 'kpi_development'        

    lp_server = lpServer(login, int(month), file_name)
    lp_server.start()
    print 'LP Server is started ....'


