from launchpadlib.launchpad import Launchpad, EDGE_SERVICE_ROOT
import os
import threading
import time

from openerprunbot.misc import *

def assert_main_thread():
    t = threading.current_thread()
    assert t.name == 'MainThread'

class LP(object):
    """
    Expose the few bits we need from launchpadlib.

    Note: any access to launchpad is done in the same thread.
    """

    def __init__(self):
        assert_main_thread()
        self.con = None
        # Mapping team name - last query time to Launchpad
        self.last_lp_call_per_team = {}

    def connect(self):
        assert_main_thread()
        #launchpad = Launchpad.login_anonymously(
        #    'openerp-runbot', 'edge', 'lpcache')
        cachedir = os.path.expanduser("~/.launchpadlib/cache")
        self.con = Launchpad.login_with(
            'openerp-runbot', EDGE_SERVICE_ROOT, cachedir,
            credentials_file='.lp_creds')

    def get_team_branches(self, team_name):
        assert_main_thread()
        try:
            team = self.con.people[team_name]
        except Exception, e: # What's the proper raised exception?
            log("WARNING: no such team:", team_name)
            return []

        default_branch_filter = {'status': ['Experimental', 'Development', 'Mature']}
        last_lp_call = self.last_lp_call_per_team.get(team_name)
        if not last_lp_call:
            team_branches = team.getBranches(**default_branch_filter)
        else:
            team_branches = team.getBranches(
                modified_since=last_lp_call,
                **default_branch_filter)
        self.last_lp_call_per_team[team_name] = time.strftime('%Y-%m-%d')
        return team_branches

    def get_branch(self, unique_name):
        assert_main_thread()
        return self.con.branches.getByUniqueName(unique_name=unique_name)

