"""
Runbot dynamic web server, to handle stickiness changes, build command and
teams and branches registration.
"""
import os
import simplejson
import threading
import urllib
import urlparse
import sys
import pdb

import openerprunbot

import logging
_logger = logging.getLogger(__name__)
runbot_state_lock = threading.Lock()

httpd = None

RUNBOT_PASSWORD = os.environ.get('RUNBOT_PASSWORD')
os.environ.pop('RUNBOT_PASSWORD', '')   # do not leak to subprocesses.
PDB_SET_TRACE_PATH = os.path.join( os.path.normpath( os.path.dirname( __file__ ) ), '..', "pdb_set_trace")

def serve(port):
    """ Serve HTTP requests via werkzeug development server.

    If werkzeug can not be imported, we fall back to wsgiref's simple_server.

    Calling this function is blocking, you might want to call it in its own
    thread.
    """

    global httpd

    interface = '127.0.0.1'
    try:
        import werkzeug.serving
        print "port",port, "interface", interface, "application", application
        httpd = werkzeug.serving.make_server(interface, port, application, threaded=True)
        openerprunbot.misc.log('HTTP service (werkzeug) running on %s:%s' % (interface, port))
    except ImportError, e:
        import wsgiref.simple_server
        httpd = wsgiref.simple_server.make_server(interface, port, application)
        openerprunbot.misc.log('HTTP service (wsgiref) running on %s:%s' % (interface, port))

    try:
        httpd.serve_forever()
    except Exception:
        _logger.exception('Cannot serve on %s:%s', interface, port)
        # TODO clean jobs?
        sys.exit(1)

def start_server(port):
    """ Call serve() in its own thread.

    The WSGI server can be shutdown with stop_server() below.
    """
    threading.Thread(name='http', target=serve, args=(port,)).start()

def stop_server():
    """ Initiate the shutdown of the WSGI server.

    The server is supposed to have been started by start_server() above.
    """
    if httpd:
        httpd.shutdown()

def read_state():
    with runbot_state_lock:
        try:
            path = os.path.join('static', 'state.runbot')
            if os.path.exists(path):
                with open(path, 'r') as f:
                    state = simplejson.loads(f.read())
                    openerprunbot.state = state

            # defaults state values
            current_saas = [2]
            if 'series' not in openerprunbot.state:
                openerprunbot.state['series'] = ['6.0', '6.1', '7.0'] + \
                                                ['saas-%d' % n for n in current_saas] + \
                                                ['trunk']
            if 'old-series' not in openerprunbot.state:
                openerprunbot.state['old-series'] = ['4.2', '5.0'] + \
                                                    ['saas-%d' % n for n in range(1, current_saas[0])]

            with open(path, 'w') as h:
                data = simplejson.dumps(openerprunbot.state, sort_keys=True, indent=2)
                h.write(data)

        except Exception, e:
            print "Exception in read_state(): ", e

# TODO lock
def _with_state(f):
    state = {}
    path = os.path.join('static', 'state.runbot')

    with runbot_state_lock:
        if os.path.exists(path):
            with open(path, 'r') as h:
                state = simplejson.loads(h.read())

        f(state)

        with open(path, 'w') as h:
            data = simplejson.dumps(state, sort_keys=True, indent=2)
            h.write(data)

    # Replace the whole dictionary at once.
    openerprunbot.state = state

def _delete(team, group):
    if team == 'openerp-dev' and group in openerprunbot.runbot.SERIES_BRANCHES:
        print 'delete', team, group.name, "ignored (core group)"
        return
    group_id = (team, group)
    group = openerprunbot.runbot.groups.get(group_id)
    if group:
        if group.sticky and False:
            print 'delete', team, group.name, "ignored (sticky)"
            return
        print 'delete', team, group.name
        
        del openerprunbot.runbot.groups[group_id]
        del openerprunbot.state['configured-branches'][team][group.name]#TODO: This line should be into after of get lock file.
        #delete from file too
        with runbot_state_lock:
            path = os.path.join('static', 'state.runbot')
            with open(path, 'w') as h:
                data = simplejson.dumps(openerprunbot.state, sort_keys=True, indent=2)
                h.write(data)
        
        for j in group.points:
            if j:
                j.stop()
        def f(state):
            # TODO check team and group are valid
            state.setdefault(team, {})
            state[team].pop(group, None)
            if not state[team]:
                del state[team]
        _with_state(f)

def _stick(team, group):
    print 'stick', team, group
    def f(state):
        # TODO check team and group are valid
        state.setdefault(team, {})
        state[team].setdefault(group, {})
        state[team][group]['sticky'] = True
    _with_state(f)

def _unstick(team, group):
    print 'unstick', team, group
    def f(state):
        # TODO check team and group are valid
        state.setdefault(team, {})
        state[team].setdefault(group, {})
        if 'sticky' in state[team][group]:
            del state[team][group]['sticky']
        if not state[team][group]:
            del state[team][group]
        if not state[team]:
            del state[team]
    _with_state(f)

def _build(team, group):
    print 'build', team, group
    openerprunbot.queue.put(('build', (team, group)))

def _register_branch(team, project, group):
    print 'register branch', team, project, group
    def f(state):
        # TODO check team and group are valid
        state.setdefault('registered-branches', [])
        state['registered-branches'].append("~%s/%s/%s" % (team, project, group))
    _with_state(f)

def _register_team(team):
    print 'register team'
    def f(state):
        # TODO check team is valid
        state.setdefault('registered-teams', []) # TODO must be a {}
        state['registered-teams'].append(team)
    _with_state(f)

def _configure_group(team_name=None, name=None, version=None, server_branch=None,
        client_web_branch=None, web_branch=None, addons_branch_0=None,
        addons_branch_1=None, addons_branch_2=None, modules=None):
    openerprunbot.misc.log('configure group '+ team_name + ':' + name )
    addons_branches = []
    for i in (addons_branch_0, addons_branch_1, addons_branch_2):
        if i:
            addons_branches.append(i)
    def f(state):
        # TODO check team is valid
        state.setdefault('configured-branches', {})
        state['configured-branches'].setdefault(team_name, {})
        state['configured-branches'][team_name].setdefault(name, {})
        state['configured-branches'][team_name][name] = dict(
            version=version,
            server_branch=server_branch,
            client_web_branch=client_web_branch,
            web_branch=web_branch,
            addons_branches=addons_branches,
            modules=modules,
        )
    _with_state(f)

def _push_series(series):
    def f(state):
        state['series'][-1:-1] = [series]         # keep trunk last
        try:
            state['old-series'].remove(series)
        except ValueError:
            pass
    _with_state(f)
    openerprunbot.queue.put(('build', ('openerp-dev', series)))

def _pop_series(series):
    def f(state):
        team = 'openerp-dev'
        group_id = (team, series)
        group = openerprunbot.runbot.groups.get(group_id)
        if group:
            del openerprunbot.runbot.groups[group_id]
            for j in group.points:
                if j:
                    j.stop()

        state.setdefault(team, {}).pop(series, None)
        if not state[team]:
            del state[team]

        state['old-series'].append(series)
        try:
            state['series'].remove(series)
        except ValueError:
            pass
    _with_state(f)

def _configure_runbot(max_branches, max_jobs):
    if max_jobs:
        openerprunbot.runbot.workers = int(max_jobs)

    # FIXME wont work until we change the way runbot manage ports
    # def f(state):
    #     if max_branches:
    #         state[''] = max_branches
    # _with_state(f)

class DictDiffer(object):
    """
    Calculate the difference between two dictionaries as:
    (1) items added
    (2) items removed
    (3) keys same in both but changed values
    (4) keys same in both and unchanged values
    """
    def __init__(self, current_dict, past_dict):
        self.current_dict, self.past_dict = current_dict, past_dict
        self.set_current, self.set_past = set(current_dict.keys()), set(past_dict.keys())
        self.intersect = self.set_current.intersection(self.set_past)
    
    def added(self):
        return self.set_current - self.intersect 
    
    def removed(self):
        return self.set_past - self.intersect
    
    def changed(self):
        return set(o for o in self.intersect if self.past_dict[o] != self.current_dict[o])
    
    def unchanged(self):
        return set(o for o in self.intersect if self.past_dict[o] == self.current_dict[o])
    
    def added_removed_changed(self):
        added = self.added()
        removed = self.removed()
        changed = self.changed()
        return added.union( removed ).union( changed )

def get_teams_groups_to_reset(new, old):
    team_dict_obj = DictDiffer(new['configured-branches'], old['configured-branches'])
    teams_names_to_reset = list( team_dict_obj.added_removed_changed() )
    team_group_to_reset = []
    for team_name_to_reset in teams_names_to_reset:
        group_dict_obj = DictDiffer(old['configured-branches'].get(team_name_to_reset, {}), new['configured-branches'].get(team_name_to_reset, {}))
        group_names_to_reset = list( group_dict_obj.added_removed_changed() )
        for group_name_to_reset in group_names_to_reset:
            team_group_to_reset.append( (team_name_to_reset,group_name_to_reset) )
    return team_group_to_reset

def _reset_state_runbot(new_state_runbot_text):
    if eval( open( PDB_SET_TRACE_PATH, "r").read() ).get('server_py_reset_state_runbot', False):
        pdb.set_trace()
    #GROUPS: TODO DELETE, DONE ADD, TODO MODIFY
    #SERIES: TODO DELETE, TODO ADD, TODO MODIFY
    #GRAL TODO: before of loads, make a eval of dict.
    new_state_runbot = simplejson.loads(new_state_runbot_text)
    #openerprunbot.state = new_state_runbot    
    ##########new_state_runbot_branches = new_state_runbot['configured-branches']
    #old_state_runbot = openerprunbot.read()
    ##########old_state_runbot_branches = openerprunbot.state['configured-branches']
    #TODO: Only reset branches with changes. DONE!
    teams_groups_to_reset = get_teams_groups_to_reset(new_state_runbot, openerprunbot.state)
    for (team_name, group_name) in teams_groups_to_reset:
        _delete(team_name, group_name)
        #TODO: Delete MP to this group too
        #group = new_state_runbot['configured-branches'].has_key( team_name ) and new_state_runbot['configured-branches'][team_name].has_key(group_name) and \
           #new_state_runbot['configured-branches'][team_name][group_name] or False
        group = new_state_runbot['configured-branches'].get( team_name, {} ).get( group_name, {} ) or False
        if group:
            addons_branch_0 = len( group['addons_branches'] ) >= 1 and group['addons_branches'][0]
            addons_branch_1 = len( group['addons_branches'] ) >= 2 and group['addons_branches'][1]
            addons_branch_2 = len( group['addons_branches'] ) >= 3 and group['addons_branches'][2]
            _configure_group(team_name=team_name, name=group_name, version=group['version'], server_branch= group['server_branch'],
                client_web_branch=group['client_web_branch'], web_branch=group['web_branch'], addons_branch_0=addons_branch_0,
                addons_branch_1=addons_branch_1, addons_branch_2=addons_branch_2, modules=group['modules'])
    
    

def application(environ, start_response):
    def send_notice(message):
        response = openerprunbot.templates.render_template('notice.html.mako', message=message)
        start_response('200 OK', [('Content-Type', 'text/html'), ('Content-Length', str(len(response)))])
        return [response]

    def _403():
        start_response('403 Forbidden', [])
        return ['']

    def check_security():
        return RUNBOT_PASSWORD and RUNBOT_PASSWORD == environ.get('HTTP_X_RUNBOT')

    if environ['REQUEST_METHOD'] == 'GET' and environ['PATH_INFO'].startswith('/hostname/'):
        hostname = environ['PATH_INFO'][len('/hostname/'):]
        response = openerprunbot.runbot.get_by_host(hostname)
        start_response('200 OK', [('Content-Type', 'application/json'), ('Content-Length', str(len(response)))])
        return [response]

    if environ['REQUEST_METHOD'] == 'POST' and environ['PATH_INFO'] == ('/a/branch'):
        length = int(environ['CONTENT_LENGTH'])
        data = urllib.unquote(environ['wsgi.input'].read(length))
        data_ = urlparse.parse_qs(data).get('branch-input',[''])[0]
        m = openerprunbot.branch_input_re.match(data_)
        if m:
            team_name = m.group(1)
            project = m.group(2)
            branch_name = m.group(3)
            _register_branch(team_name, project, branch_name)
            response = data_ + ' is registered. It will be picked by the Runbot at the next build iteration (about every two minutes).'
            return send_notice(response)
            
    if environ['REQUEST_METHOD'] == 'POST' and environ['PATH_INFO'] == ('/a/state_runbot'):
        openerprunbot.misc.log('HTTP POST state_runbot')
        length = int(environ['CONTENT_LENGTH'])
        data = urllib.unquote(environ['wsgi.input'].read(length))
        state_runbot_textarea = urlparse.parse_qs(data).get('name_state_runbot_textarea', [{}])[0]
        _reset_state_runbot(state_runbot_textarea)
        return send_notice("Click en el boton de state_runbot write")

    if environ['REQUEST_METHOD'] == 'POST' and environ['PATH_INFO'] == ('/a/team'):
        length = int(environ['CONTENT_LENGTH'])
        data = urllib.unquote(environ['wsgi.input'].read(length))
        data_ = urlparse.parse_qs(data).get('team-input',[''])[0]
        if data_:
            _register_team(data_)
            response = data_ + ' is registered. It will be picked by the Runbot at the next build iteration (about every two minutes).'
            return send_notice(response)

    if environ['REQUEST_METHOD'] == 'POST' and environ['PATH_INFO'] == ('/a/configure-build'):
        length = int(environ['CONTENT_LENGTH'])
        data = urllib.unquote(environ['wsgi.input'].read(length))
        data_ = {}
        for n in (
            "team_name", "name", "version", "server_branch",
            "client_web_branch", "web_branch",
            "addons_branch_0", "addons_branch_1", "addons_branch_2", "modules"):
            data_[n] = urlparse.parse_qs(data).get(n,[''])[0]
        if (data_['version'] in openerprunbot.runbot.SERIES_BRANCHES) and \
            data_['team_name'] and \
            data_['name'] and \
            data_['server_branch'] and \
            (data_['web_branch'] or data_['client_web_branch']) and \
            data_['addons_branch_0']:
            _configure_group(**data_)
            response = data_['name'] + ' is registered. It will be picked by the Runbot at the next build iteration (about every two minutes).'
            return send_notice(response)

    if environ['REQUEST_METHOD'] == 'POST' and environ['PATH_INFO'] == '/a/push-series':
        if not check_security():
            return _403()
        length = int(environ['CONTENT_LENGTH'])
        wsgiinput = urllib.unquote(environ['wsgi.input'].read(length))
        series = urlparse.parse_qs(wsgiinput).get('series', [''])[0]
        _push_series(series)
        msg = '`%s` series added' % (series,)
        return send_notice(msg)

    if environ['REQUEST_METHOD'] == 'POST' and environ['PATH_INFO'] == '/a/pop-series':
        if not check_security():
            return _403()
        length = int(environ['CONTENT_LENGTH'])
        wsgiinput = urllib.unquote(environ['wsgi.input'].read(length))
        series = urlparse.parse_qs(wsgiinput).get('series', [''])[0]
        _pop_series(series)
        msg = '`%s` series removed' % (series,)
        return send_notice(msg)

    if environ['REQUEST_METHOD'] == 'POST' and environ['PATH_INFO'] == '/a/configure':
        if not check_security():
            return _403()
        length = int(environ['CONTENT_LENGTH'])
        wsgiinput = urllib.unquote(environ['wsgi.input'].read(length))
        qs = urlparse.parse_qs(wsgiinput)
        max_branches = qs.get('max_branches', [''])[0]
        max_jobs = qs.get('max_jobs', [''])[0]
        _configure_runbot(max_branches, max_jobs)
        return send_notice('Runbot reconfigured')

    if environ['REQUEST_METHOD'] == 'POST' and environ['PATH_INFO'] == ('/a'):
        query = urlparse.parse_qs(environ['QUERY_STRING'])

        if query.get('team'):
            team = query['team'][0]

            if query.get('deletegroup'):
                group = query['deletegroup'][0]
                _delete(team, group)
                response = group + ' is gone forever!'
                return send_notice(response)

            if query.get('stick'):
                group = query['stick'][0]
                _stick(team, group)
                response = group + ' is now sticky'
                return send_notice(response)

            if query.get('unstick'):
                group = query['unstick'][0]
                _unstick(team, group)
                response = group + ' is now non-sticky'
                return send_notice(response)

            if query.get('build'):
                group = query['build'][0]
                _build(team, group)
                response = '`' + group + '` will be built.'
                return send_notice(response)

    message = 'Are you lost?\n\n\n(^____^)\n'
    response = openerprunbot.templates.render_template('notice.html.mako', message=message)
    start_response('404 Not Found', [('Content-Type', 'text/html'), ('Content-Length', str(len(response)))])
    return [response]

if __name__ == '__main__':
    serve(8999)
