"""
Mockup Launchpad library.
"""

import threading

def assert_main_thread():
    t = threading.current_thread()
    assert t.name == 'MainThread'

class LP(object):
    """
    Expose the few bits we need from launchpadlib.

    Note: any access to launchpad is done in the same thread.
    """

    def __init__(self):
        assert_main_thread()

    def connect(self):
        assert_main_thread()

    def get_team_branches(self, team_name):
        assert_main_thread()
        if team_name in KNOWN_TEAM_BRANCHES:
            return map(Branch, KNOWN_TEAM_BRANCHES[team_name])
        else:
            log("WARNING: no such team:", team_name)
            return []

    def get_branch(self, unique_name):
        assert_main_thread()
        if unique_name in KNOWN_BRANCHES:
            return Branch(unique_name)
        else:
            return None

class Branch(object):

    def __init__(self, unique_name):
        self.unique_name = unique_name
        self.name = unique_name.split('/')[2]
        self.date_last_modified = 'hello' # TODO
        self.revision_count = KNOWN_BRANCHES[unique_name]

    def getMergeProposals(self, **filters):
        return []

KNOWN_BRANCHES = {
    '~openerp/openobject-server/6.0': 3547,
    '~openerp/openobject-server/6.1': 4154,
    '~openerp/openobject-server/trunk': 3856,
    '~openerp/openobject-addons/6.0': 4946,
    '~openerp/openobject-addons/6.1': 6768,
    '~openerp/openobject-addons/trunk': 5890,
    '~openerp/openobject-client-web/6.0': 4741,
    '~openerp/openerp-web/6.1': 2311,
    '~openerp/openerp-web/trunk': 1624,
}

